/*******************************************************************************
*   Filename:       bsp_iwdg.c
*   Revised:        All copyrights reserved to Roger-WY.
*   Date:           2016-06-29
*   Revision:       v1.0
*   Writer:	        Roger-WY
*
*   Description:    STM32内部独立看门狗驱动模块
*
*   Notes:
*   独立看门狗工作原理：在键值寄存器（IWDG_KR）中写入0XCCCC，开始启用独立看门狗，
*   此时计数器开始从其复位值OXFFF递减计数，当计数器计数到末尾0X000的时候，会产生
*   一个复位信号（IWDG_RESET），无论何时，只要寄存器IWDG_KR中被写入0XAAAA，IWDG_RLR
*   中的值就会被重新加载到计数器中从而避免产生看门狗复位。
*******************************************************************************/


/*******************************************************************************
* INCLUDES
*/
#include    "bsp_iwdg.h"


static uint8_t s_ucIwdgMode  = BSP_WDT_MODE_INT;    // 0:禁止；1：外部看门狗；2：内部看门狗；3：同时使用内部和外部看门狗;

//============================================================================//

/*******************************************************************************
 * 名    称： Bsp_IwdgFeed
 * 功    能： 独立看门狗喂狗（看门狗复位）
 * 入口参数： 无
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
void  bsp_IwdgFeed(void)
{
    if(s_ucIwdgMode == BSP_WDT_MODE_NONE)
        return;

    /***********************************************
    * 描述：内部看门狗喂狗 Reload IWDG counter
    */
    if((s_ucIwdgMode == BSP_WDT_MODE_INT) || (s_ucIwdgMode == BSP_WDT_MODE_ALL))
    {
        WDT_Clear(); 
    }
    /***********************************************
    * 描述：外部看门狗喂狗
    */
    if((s_ucIwdgMode == BSP_WDT_MODE_EXT) || (s_ucIwdgMode == BSP_WDT_MODE_ALL))
    {
        //进入临界区
        //PORT_SetBits(GPIO_WDT_PORT, BSP_WDT_PIN);
        //Delay_Nus(10);
        //PORT_ResetBits(GPIO_WDT_PORT, BSP_WDT_PIN);
        //推出临界区
    }
}

/*******************************************************************************
 * 名    称： Bsp_IwdgGetMode
 * 功    能： 获取用了上面狗（内部的？外部的？还是两个都用？）
 * 入口参数： 无
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
uint8_t  bsp_IwdgGetMode(void)
{
    return s_ucIwdgMode;
}

/*******************************************************************************
 * 名    称： bsp_IwdgInit
 * 功    能： 独立看门狗初始化
 * 入口参数： 0:禁止；1：外部看门狗；2：内部看门狗；3：同时使用内部和外部看门狗;
 * 出口参数： 0：初始化成功 1：初始化失败
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
uint8_t bsp_IwdgInit(uint8_t mode)
{
    s_ucIwdgMode = mode;


    if(mode == BSP_WDT_MODE_NONE)     //禁止狗
        return 0;

    if((mode == BSP_WDT_MODE_INT) || (mode == BSP_WDT_MODE_ALL))         //使用内部狗或者内部外部狗一起用
    {
        WDT_InitTypeDef	WDT_InitStructure;					//结构定义

        WDT_InitStructure.WDT_Enable     = ENABLE;					//中断使能   ENABLE或DISABLE
        WDT_InitStructure.WDT_IDLE_Mode  = WDT_IDLE_STOP;		//IDLE模式是否停止计数		WDT_IDLE_STOP,WDT_IDLE_RUN
        WDT_InitStructure.WDT_PS         = WDT_SCALE_16;		//看门狗定时器时钟分频系数		WDT_SCALE_2,WDT_SCALE_4,WDT_SCALE_8,WDT_SCALE_16,WDT_SCALE_32,WDT_SCALE_64,WDT_SCALE_128,WDT_SCALE_256
        WDT_Inilize(&WDT_InitStructure);					//初始化
    }
    if((mode == BSP_WDT_MODE_EXT) || (mode == BSP_WDT_MODE_ALL))         //使用外狗或者内狗外狗一起用
    {

    }

    return 1;
}

/*******************************************************************************
*              end of file                                                    *
*******************************************************************************/
