/*******************************************************************************
*   Filename:       bsp_key.c
*   Revised:        All copyrights reserved to Roger-WY.
*   Date:           2016-06-29
*   Revision:       v1.0
*   Writer:	        Roger-WY
*
*   Description:   独立按键驱动模块
*
*   Notes:         扫描独立按键，具有软件滤波机制，具有按键FIFO。可以检测如下事件：
*				   (1) 按键按下
*				   (2) 按键弹起
*				   (3) 长按键
*				   (4) 长按时自动连发
*
*
*******************************************************************************/
#include "bsp_key.h"

/*
	如果用于其它硬件，请修改GPIO定义和 IsKeyDown1 - IsKeyDown8 函数

	如果用户的按键个数小于10个，你可以将多余的按键全部定义为和第1个按键一样，并不影响程序功能
	#define KEY_COUNT    10	  这个在 bsp_key.h 文件中定义
*/


/* 按键口对应的RCC时钟 */
#define RCC_ALL_KEY 	( )

#define GPIO_PORT_K1    (GPIO_P1)
#define GPIO_PIN_K1	    (GPIO_Pin_3)
#define GPIO_K1	        (P13)



//----------------------------------------------------------------------------//
static KEY_T s_tBtn[KEY_COUNT];
static KEY_FIFO_T s_tKey;		/* 按键FIFO变量,结构体 */

static void bsp_InitKeyVar(void);
static void bsp_InitKeyHard(void);
static void bsp_DetectKey(uint8_t i);

//============================================================================//

/*******************************************************************************
 * 名    称： IsKeyDownX
 * 功    能： 判断按键是否按下
 * 入口参数： 无
 * 出口参数： 返回值1 表示按下，0表示未按下
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
static uint8_t IsKeyDown1(void) {if (GPIO_K1 == 0) return 1;else return 0;}



/*******************************************************************************
 * 名    称： Bsp_KeyInit
 * 功    能： 初始化按键. 该函数被 Bsp_Init() 调用。
 * 入口参数： 无
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
void bsp_KeyInit(void)
{
	bsp_InitKeyVar();		/* 初始化按键变量 */
	bsp_InitKeyHard();		/* 初始化按键硬件 */
}

/*******************************************************************************
 * 名    称： Bsp_PutKey
 * 功    能： 将1个键值压入按键FIFO缓冲区。可用于模拟一个按键。
 * 入口参数：  _KeyCode : 按键代码
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
void bsp_PutKey(uint8_t _KeyCode)
{
	s_tKey.Buf[s_tKey.Write] = _KeyCode;

	if (++s_tKey.Write  >= KEY_FIFO_SIZE) {
		s_tKey.Write = 0;
	}
}

/*******************************************************************************
 * 名    称： Bsp_GetKey
 * 功    能： 从按键FIFO缓冲区读取一个键值。
 * 入口参数： 无
 * 出口参数： 按键代码
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
uint8_t bsp_GetKey(void)
{
	uint8_t ret;

	if (s_tKey.Read == s_tKey.Write) {
		return KEY_NONE;
	} else {
		ret = s_tKey.Buf[s_tKey.Read];

		if (++s_tKey.Read >= KEY_FIFO_SIZE) {
			s_tKey.Read = 0;
		}
		return ret;
	}
}

/*******************************************************************************
 * 名    称： Bsp_GetKey2
 * 功    能： 从按键FIFO缓冲区读取一个键值。独立的读指针。
 * 入口参数： 无
 * 出口参数： 按键代码
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
uint8_t bsp_GetKey2(void)
{
	uint8_t ret;

	if (s_tKey.Read2 == s_tKey.Write) {
		return KEY_NONE;
	} else {
		ret = s_tKey.Buf[s_tKey.Read2];

		if (++s_tKey.Read2 >= KEY_FIFO_SIZE) {
			s_tKey.Read2 = 0;
		}
		return ret;
	}
}

/*******************************************************************************
 * 名    称： Bsp_GetKeyState
 * 功    能： 读取按键的状态
 * 入口参数：  _ucKeyID : 按键ID，从0开始
 * 出口参数： 1 表示按下， 0 表示未按下
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
uint8_t bsp_GetKeyState(KEY_ID_E _ucKeyID)
{
	return s_tBtn[_ucKeyID].State;
}

/*******************************************************************************
 * 名    称： Bsp_SetKeyParam
 * 功    能： 设置按键参数
 * 入口参数： _ucKeyID : 按键ID，从0开始
 *			  _LongTime : 长按事件时间
 *			  _RepeatSpeed : 连发速度
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
void bsp_SetKeyParam(uint8_t _ucKeyID, uint16_t _LongTime, uint8_t  _RepeatSpeed)
{
	s_tBtn[_ucKeyID].LongTime    = _LongTime;			/* 长按时间 0 表示不检测长按键事件 */
	s_tBtn[_ucKeyID].RepeatSpeed = _RepeatSpeed;		/* 按键连发的速度，0表示不支持连发 */
	s_tBtn[_ucKeyID].RepeatCount = 0;					/* 连发计数器 */
}

/*******************************************************************************
 * 名    称： Bsp_ClearKey
 * 功    能： 清空按键FIFO缓冲区
 * 入口参数： 无
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
void bsp_ClearKey(void)
{
	s_tKey.Read = s_tKey.Write;
}

/*******************************************************************************
 * 名    称： bsp_InitKeyHard
 * 功    能： 配置按键对应的GPIO
 * 入口参数： 无
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
static void bsp_InitKeyHard(void)
{
    GPIO_InitTypeDef	GPIO_InitStructure;		//结构定义
	GPIO_InitStructure.Pin  = GPIO_PIN_K1;		//指定要初始化的IO,
	GPIO_InitStructure.Mode = GPIO_PullUp;		//指定IO的输入或输出方式,GPIO_PullUp,GPIO_HighZ,GPIO_OUT_OD,GPIO_OUT_PP
	GPIO_Inilize(GPIO_PORT_K1,&GPIO_InitStructure);	//初始化
}

/*******************************************************************************
 * 名    称： bsp_InitKeyVar
 * 功    能： 初始化按键变量
 * 入口参数： 无
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注：
 *******************************************************************************/
static void bsp_InitKeyVar(void)
{
	uint8_t i;

	/* 对按键FIFO读写指针清零 */
	s_tKey.Read = 0;
	s_tKey.Write = 0;
	s_tKey.Read2 = 0;

	/* 给每个按键结构体成员变量赋一组缺省值 */
	for (i = 0; i < KEY_COUNT; i++) {
		s_tBtn[i].LongTime = KEY_LONG_TIME;			/* 长按时间 0 表示不检测长按键事件 */
		s_tBtn[i].Count = KEY_FILTER_TIME / 2;		/* 计数器设置为滤波时间的一半 */
		s_tBtn[i].State = 0;							/* 按键缺省状态，0为未按下 */
		//s_tBtn[i].KeyCodeDown = 3 * i + 1;				/* 按键按下的键值代码 */
		//s_tBtn[i].KeyCodeUp   = 3 * i + 2;				/* 按键弹起的键值代码 */
		//s_tBtn[i].KeyCodeLong = 3 * i + 3;				/* 按键被持续按下的键值代码 */
		s_tBtn[i].RepeatSpeed = 0;						/* 按键连发的速度，0表示不支持连发 */
		s_tBtn[i].RepeatCount = 0;						/* 连发计数器 */
	}

	/* 如果需要单独更改某个按键的参数，可以在此单独重新赋值 */
	/* 比如，我们希望按键1按下超过1秒后，自动重发相同键值 */


	/* 判断按键按下的函数 */
	s_tBtn[0].IsKeyDownFunc = IsKeyDown1;
//	s_tBtn[1].IsKeyDownFunc = IsKeyDown2;
//	s_tBtn[2].IsKeyDownFunc = IsKeyDown3;
//	s_tBtn[3].IsKeyDownFunc = IsKeyDown4;
//	s_tBtn[4].IsKeyDownFunc = IsKeyDown5;
//	s_tBtn[5].IsKeyDownFunc = IsKeyDown6;

}

/*******************************************************************************
 * 名    称： bsp_DetectKey
 * 功    能： 检测一个按键。
 * 入口参数：  按键结构变量指针
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注： 非阻塞状态，必须被周期性的调用。
 *******************************************************************************/
static void bsp_DetectKey(uint8_t i)
{
	KEY_T *pBtn;

	/*
		如果没有初始化按键函数，则报错
		if (s_tBtn[i].IsKeyDownFunc == 0)
		{
			printf("Fault : DetectButton(), s_tBtn[i].IsKeyDownFunc undefine");
		}
	*/

	pBtn = &s_tBtn[i];
	if (pBtn->IsKeyDownFunc()) {
		if (pBtn->Count < KEY_FILTER_TIME) {
			pBtn->Count = KEY_FILTER_TIME;
		} else if(pBtn->Count < 2 * KEY_FILTER_TIME) {
			pBtn->Count++;
		} else {
			if (pBtn->State == 0) {
				pBtn->State = 1;

				/* 发送按钮按下的消息 */
				bsp_PutKey((uint8_t)(3 * i + 1));
			}

			if (pBtn->LongTime > 0) {
				if (pBtn->LongCount < pBtn->LongTime) {
					/* 发送按钮持续按下的消息 */
					if (++pBtn->LongCount == pBtn->LongTime) {
						/* 键值放入按键FIFO */
						bsp_PutKey((uint8_t)(3 * i + 3));
					}
				} else {
					if (pBtn->RepeatSpeed > 0) {
						if (++pBtn->RepeatCount >= pBtn->RepeatSpeed) {
							pBtn->RepeatCount = 0;
							/* 常按键后，每隔10ms发送1个按键 */
							bsp_PutKey((uint8_t)(3 * i + 1));
						}
					}
				}
			}
		}
	} else {
		if(pBtn->Count > KEY_FILTER_TIME) {
			pBtn->Count = KEY_FILTER_TIME;
		} else if(pBtn->Count != 0) {
			pBtn->Count--;
		} else {
			if (pBtn->State == 1) {
				pBtn->State = 0;

				/* 发送按钮弹起的消息 */
				bsp_PutKey((uint8_t)(3 * i + 2));
			}
		}

		pBtn->LongCount = 0;
		pBtn->RepeatCount = 0;
	}
}

/*******************************************************************************
 * 名    称： Bsp_KeyScan
 * 功    能： 扫描所有按键。非阻塞，被systick中断周期性的调用
 * 入口参数： 无
 * 出口参数： 无
 * 作　　者： Roger-WY.
 * 创建日期： 2015-06-25
 * 修    改：
 * 修改日期：
 * 备    注： 非阻塞，被bsp.c中Bsp_RunPer10ms()函数周期性的调用
 *******************************************************************************/
void bsp_KeyScan(void)
{
	uint8_t i;

	for (i = 0; i < KEY_COUNT; i++) {
		bsp_DetectKey(i);
	}
}

/*****************************  (END OF FILE) *********************************/
